from ngsolve import *
import numpy as np

pi = np.pi
sin = np.sin

mesh = Mesh("cube.vol")
names = [ "bottom", "top", "left", "right", "front", "back", "inner" ]
for i, name in enumerate(names):
    mesh.ngmesh.SetBCName(i, name)
print (mesh.GetBoundaries())
    

B0 = 1.
mu0 = 4. * pi * 1.0e-7
mus = {'air': 1., 'core': 1000.}
sigmas = {'air': 10., 'core': 2e6}

f = 1.0 # Hz
omega = 2. * pi * f
T = 1. / f  # s
dt = T * 5./360.

# Finite element space
V = HCurl(mesh, order = 1, dirichlet = "right|back")

# Trial function
u = V.TrialFunction()
# Test function
v = V.TestFunction()

uh = GridFunction(V)

# permeabitlity
muValues = {mat: mu0*mus[mat] for mat in mesh.GetMaterials()}
mu = CoefficientFunction([muValues[mat]
                        for mat in mesh.GetMaterials()])

# conductivity
sigmaValues = {mat: sigmas[mat] for mat in mesh.GetMaterials()}
sigma = CoefficientFunction([sigmaValues[mat]
                            for mat in mesh.GetMaterials()])

a = BilinearForm(V, symmetric=True)
a += SymbolicBFI(1 / mu * InnerProduct(curl(u), curl(v))
                + sigma / dt * InnerProduct(u, v))

with TaskManager():
    c = Preconditioner(a, "bddc")
    a.Assemble()

t = 0
Draw (uh)

with open("NGSolve_A_1Hz.dat", 'w') as fd:
    fd.write("#x y z Bx By Bz\n")
    while t < 1: #  1*T:
        uh_old = uh
        dirvalues = { "right" : (0,25e-3*B0*x*sin(omega*t),0), "back" : (-25e-3*y*B0*sin(omega * t),0,0) }
        dirlist = [ dirvalues[bcname] if bcname in dirvalues.keys() else (0,0,0) \
                        for bcname in mesh.GetBoundaries() ]
        coeff_dirichlet = CoefficientFunction( dirlist )
        # coeff_dirichlet = CoefficientFunction((
        # CoefficientFunction([0,0,0,0,0,-25e-3*y*B0*sin(omega * t)]), 
        # CoefficientFunction([0,0,0,25e-3*B0*x*sin(omega * t),0,0]), 
        # CoefficientFunction([0,0,0,0,0])))
        uh.Set(coeff_dirichlet, BND)
        
        # Linearform
        f = LinearForm(V)
        f += SymbolicLFI(sigma / dt * InnerProduct(uh_old, v))


        # Solver
        with TaskManager():
            f.Assemble()
            bvp = BVP(bf=a, lf=f, gf=uh, pre=c)
            bvp.Do()

        t += dt

        for i_x in np.linspace(0., 10e-3, num=100):
            A = uh(i_x, 10e-3, 0.)
            fd.write("%.15e %.15e %.15e %.15e %.15e %.15e\n" % (i_x,10e-3,0.,A[0],A[1],A[2]))
        fd.write("\n")
        
        Redraw()
